"""
Modified pytest discovery classes.

This is so you can use noseOfYeti to have nested describes without running the
same tests multiple times.

It achieves this by making classes have a ModifiedInstance object that discards
any collected tests that aren't on that class.
"""
from noseOfYeti.tokeniser.spec_codec import register

from _pytest.unittest import UnitTestCase
from unittest import mock
import inspect
import pytest


def pytest_configure():
    register()


@pytest.hookimpl(hookwrapper=True)
def pytest_pycollect_makeitem(collector, name, obj):
    """Make sure we can have nested noseOfYeti describes"""
    outcome = yield
    res = outcome.get_result()

    if res is None:
        return

    if not hasattr(res, "obj"):
        is_noy_on_obj = getattr(obj, "is_noy_spec", False)
        is_noy_on_class = getattr(obj.__class__, "is_noy_spec", False)
        if not is_noy_on_obj and not is_noy_on_class:
            return

    original_item_collect = res.collect

    if isinstance(res, UnitTestCase):

        def collect():
            yield from filter_collection(original_item_collect(), res.obj)

    else:

        def collect():
            res.session._fixturemanager.parsefactories(res)
            yield from filter_collection(original_item_collect(), res.obj)

    mock.patch.object(res, "collect", collect).start()


def filter_collection(collected, obj):
    for thing in collected:
        if not isinstance(thing, pytest.Function):
            yield thing
            continue

        try:
            isclass = inspect.isclass(obj)
            if not isclass:
                obj = obj.__class__
        except Exception:
            pass

        if obj.__dict__.get("__only_run_tests_in_children__"):
            # Only run these tests in the children, not in this class itself
            continue

        if thing.obj.__name__ in obj.__dict__:
            yield thing
        else:
            method_passed_down = any(
                thing.obj.__name__ in superkls.__dict__
                and getattr(superkls, "__only_run_tests_in_children__", False)
                for superkls in obj.__bases__
            )
            if method_passed_down:
                yield thing



from noseOfYeti.tokeniser.spec_codec import codec

from pylama.config import LOGGER
from pylama.errors import Error
from pylama.main import shell

from unittest import mock
import codecs
import re


try:
    from pylama.core import CodeContext
except ImportError:
    CodeContext = None

spec_codec = codec()

regexes = {
    "position": re.compile(r"line (\d+), column (\d+)"),
    "encoding": re.compile(r"#\s*coding\s*:\s*spec"),
}


def translate(path: str, contents: str) -> str:
    lines = contents.split("\n")
    pattern = "%(filename)s:%(lnum)s:%(col)s: [%(type)s] %(text)s"

    if contents and regexes["encoding"].match(lines[0]):
        contents = spec_codec.translate(contents)

        if "--- internal spec codec error ---" in lines[-1]:
            found = []

            useful = False
            for line in lines[1:-2]:
                if useful or "SyntaxError:" in line:
                    found.append(line)
                    useful = True

            line = 0
            column = 1
            for line in found:
                m = regexes["position"].search(line)
                if m:
                    line, column = m.groups()
                    break

            msg = "\t".join(found)
            err = Error(linter="pylama", col=column, lnum=line, text=msg, filename=path)
            LOGGER.warning(pattern, err._info)

    return contents


def run_old_pylama(spec_codec):
    original = CodeContext.__enter__

    def interpret(self):
        ret = original(self)
        if not self.code:
            return ret

        self.code = translate(self.path, self.code)
        return ret

    with mock.patch.object(CodeContext, "__enter__", interpret):
        shell()


def run_pylama():
    spec_codec.register()

    if CodeContext is None:

        def better_read(filename: str) -> str:
            with codecs.open(filename) as fle:
                return translate(filename, fle.read())

        with mock.patch("pylama.context.read", better_read):
            return shell()
    else:
        run_old_pylama(spec_codec)



from noseOfYeti.tokeniser.spec_codec import codec

from mypy.plugin import Plugin

spec_codec = codec()


class NoyPlugin(Plugin):
    pass


def plugin(version: str):
    spec_codec.register()
    return NoyPlugin



from setuptools import setup, find_packages
from noseOfYeti import VERSION

# fmt: off

setup(
      name = 'noseOfYeti'
    , version = VERSION
    , packages = find_packages(include="noseOfYeti.*", exclude=["tests*"])

    , classifiers =
      [ 'Intended Audience :: Developers'
      , 'Programming Language :: Python'
      , 'Topic :: Software Development :: Documentation'
      , 'Topic :: Software Development :: Testing'
      ]

    , author = 'Stephen Moore'
    , license = 'MIT'
    , keywords = 'bdd rspec spec'
    , author_email = 'delfick755@gmail.com'

    , url = "https://github.com/delfick/nose-of-yeti"
    , description = 'A custom pyton codec that provides an RSpec style dsl for python'
    , long_description = open("README.rst").read()

    , extras_require =
      { 'tests':
        [ "asynctest==0.13.0"
        , "pytest>=7.0.1"
        , "alt-pytest-asyncio==0.6.0"
        , "pytest-helpers-namespace==2021.4.29"
        ]
      }

    , entry_points =
      { 'console_scripts' :
        [ 'run_noseOfYeti_tests = noseOfYeti:run_pytest'
        , 'noy_pylama = noseOfYeti.plugins.pylama:run_pylama'
        ]
      , 'nose.plugins':
        [ 'noseOfYeti = noseOfYeti.plugins.nosetests:Plugin'
        ]
      , "pytest11": ["nose_of_yeti = noseOfYeti.plugins.pytest"]
      , 'pyls': ['pyls_noy = noseOfYeti.plugins.pyls']
      }
    )

# fmt: on
